#include "pbc_nxx.h"



NXXPBC::NXXPBC(const string in_load_file_name,
		  const string in_save_file_name,
		  int in_xdim,int in_ydim,int in_zdim,
		  double in_dx,double in_dy,double in_dz,
          double in_error)
		  :load_file_name(in_load_file_name),
		  save_file_name(in_save_file_name),
		  xdim(in_xdim),ydim(in_ydim),zdim(in_zdim),xydim(xdim*ydim),
     		  totalsize(xydim*zdim),dx(in_dx),dy(in_dy),dz(in_dz),
		  Nxx(NULL),Nxy(NULL),Nxz(NULL),Nyy(NULL),Nyz(NULL),Nzz(NULL),
          error(in_error)			
{
	Nxx=new double[totalsize];
	Nyy=new double[totalsize];
	Nzz=new double[totalsize];
	Nxy=new double[totalsize];
	Nxz=new double[totalsize];
	Nyz=new double[totalsize];
}

void NXXPBC::ReleaseMemory() const
{ 
  if(Nxx!=NULL) { delete[] Nxx; Nxx=NULL; }
  if(Nyy!=NULL) { delete[] Nyy; Nyy=NULL; }
  if(Nzz!=NULL) { delete[] Nzz; Nzz=NULL; }
  if(Nxy!=NULL) { delete[] Nxy; Nxy=NULL; }
  if(Nxz!=NULL) { delete[] Nxz; Nxz=NULL; }
  if(Nyz!=NULL) { delete[] Nyz; Nyz=NULL; }
  xdim=ydim=zdim=xydim=totalsize=0;
  dx=dy=dz=0;
  Lcx=Lcy=0;
  error=1e-10;
}


int NXXPBC::getOrder(int i,int j,int k) const{
	return k*xydim+j*xdim+i;
}


bool NXXPBC::LoadNXX(const String source){
	char *filename=new char[source.length()+1];  
	char *str = new char[2048];

	int Nx=0,Ny=0,Nz=0,skip_n=0;
	float dx_dz=0.0,dy_dz=0.0;

	strcpy(filename,source.c_str());
	
	FILE *fp = fopen(filename, "r");
	if (fp == NULL) {
		return false;
	}

	fscanf(fp,"Nx=%d  Ny=%d  Nz=%d  dx/dz=%f  dy/dz=%f  Lines to skip:%d\n",&Nx,&Ny,&Nz,&dx_dz,&dy_dz,&skip_n);

	//more conditions should be consider,maybe later 
	if(Nx*Ny*Nz!=totalsize){
		fclose(fp);
		return false;
	}

	for(int i=0;i<skip_n;i++){
		fgets(str, 2048, fp);
	}
	
	int p=0;
	while(!feof(fp)){
		fscanf(fp,"%lf%lf%lf%lf%lf%lf\n",&Nxx[p],&Nyy[p],&Nzz[p],&Nxy[p],&Nxz[p],&Nyz[p]);
		p++;
	}
	
	if(p!=totalsize){
		fclose(fp);
		return false;
	} 
	
	delete[] str;
	delete[] filename;
	fclose(fp);
	
	return true;
}

void NXXPBC::SaveNXX(const String target){

	char *filename=new char[target.length()+1];  
	strcpy(filename,target.c_str());
	FILE *fp = fopen(filename, "w");
	if (fp == NULL) {
		return ;
	}
	
	fprintf(fp, "Nx=%d  Ny=%d  Nz=%d  dx/dz=%f  dy/dz=%f  Lines to skip:%d\n",xdim,ydim,zdim,dx/dz,dy/dz,7);

	fprintf(fp, "The Sum of Nxx: %0.15e\nThe Sum of Nyy: %0.15e\nThe Sum of Nzz: %0.15e\nThe Sum of Nxy: %0.15e\nThe Sum of Nxz: %0.15e\nThe Sum of Nyz: %0.15e\n",
				Sum(totalsize,Nxx),Sum(totalsize,Nyy),Sum(totalsize,Nzz),
				Sum(totalsize,Nxy),Sum(totalsize,Nxz),Sum(totalsize,Nyz));
	fprintf(fp, "          Nxx            ");
	fprintf(fp, "          Nyy            ");
	fprintf(fp, "          Nzz            ");
	fprintf(fp, "          Nxy            ");
	fprintf(fp, "          Nxz            ");
	fprintf(fp, "          Nyz\n");
		
	int p=0;
	for(int k=0;k<zdim;k++){
		for(int j=0;j<ydim;j++){
			for(int i=0;i<xdim;i++){
		    p=k*xydim+j*xdim+i;
		    fprintf(fp, "%+0.15e   %+0.15e   %+0.15e   %+0.15e   %+0.15e   %+0.15e\n",
					Nxx[p],Nyy[p],Nzz[p],Nxy[p],Nxz[p],Nyz[p]);
			}
		}
	}
	
	
	
	delete[] filename;
  fclose(fp);

}

void NXXPBC::CalcNXX() {

	if(load_file_name.length()!=0){
	   if(LoadNXX(load_file_name)){
		if(save_file_name.length()!=0&&save_file_name!=load_file_name){
	        SaveNXX(save_file_name);
	        }  
	  return;
	  }
	}
	
//	Lcx=findLc(xdim);
//	Lcy=findLc(ydim);
	//printf("xdim=%d  ydim=%d  zdim=%d\n",xdim,ydim,zdim);
   	
	double x,y,z;
	double dx_dz=dx/dz;
	double dy_dz=dy/dz;
	int p=0;
    int gxx,gyy,gzz;
    gxx=FindG(xx,dx_dz*dy_dz,xdim*dx_dz,ydim*dy_dz);
    gyy=FindG(yy,dx_dz*dy_dz,xdim*dx_dz,ydim*dy_dz);
    gzz=FindG(zz,dx_dz*dy_dz,xdim*dx_dz,ydim*dy_dz);
    printf("gxx=%d  gyy=%d  gzz=%d\n",gxx,gyy,gzz);
	for(int k=0;k<zdim;k++){
		for(int j=0;j<ydim;j++){
			for(int i=0;i<xdim;i++){
		//	printf("i=%d  j=%d  k=%d\n",i,j,k);
			x=i*dx_dz;
			y=j*dy_dz;
			z=k;
			p=k*xydim+j*xdim+i;
			Nxx[p]=getNXXPBC(xx,gxx,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);
			Nyy[p]=getNXXPBC(yy,gyy,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);
			//Nzz[p]=getNXXPBC(zz,Lcx,Lcy,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);
			Nzz[p]=-(Nxx[p]+Nyy[p]);
			Nxy[p]=getNXXPBC(xy,gxx,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);
			Nxz[p]=getNXXPBC(xz,gxx,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);
			Nyz[p]=getNXXPBC(yz,gxx,xdim,ydim,x,y,z,dx_dz,dy_dz,1.0);	
			//printf("%0.15e\n",Nxz[p])	;	
			}
		}
	}
	
	Nzz[0]+=1.0;
	

		if(save_file_name.length()!=0){
	        SaveNXX(save_file_name);
	        }  

}


double NXXPBC::getNXX(enum Type_Nij type,int i,int j,int k){
	int p=k*xydim+j*xdim+i;
	switch(type){
		case xx:return Nxx[p];
		case yy:return Nyy[p];
		case zz:return Nzz[p];
		case xy:return Nxy[p];
		case xz:return Nxz[p];
		case yz:return Nyz[p];
	}
}



int NXXPBC::FindG(enum Type_Nij type,double v,double Tx,double Ty) const{
 double tmp; 
  switch(type){
        case xy:
        case xz:
        case yz:
		case xx:
           tmp=v/(4*PI*(Tx*Tx)*sqrt(Tx*Tx+Ty*Ty)*error);
           return (int)pow(tmp,1/3.0);
		case yy:
           tmp=v/(4*PI*(Ty*Ty)*sqrt(Tx*Tx+Ty*Ty)*error);
           return (int)pow(tmp,1/3.0);
		case zz:
		   tmp=v*sqrt(Tx*Tx+Ty*Ty)/(4*PI*(Tx*Ty*Tx*Ty)*error);
           return (int)pow(tmp,1/3.0);
	}
}


double NXXPBC::Sum(int n,double *array) const{
   double *tmp=new double[n];
   for(int i=0;i<n;i++){
     tmp[i]=array[i];
    }
   double result= AccurateSum(n,tmp);
   delete tmp;
   return result;
}
