#include <math.h>
#include <stdlib.h>   
#include "pbc_util.h"
#include "demagcoef.h"

#define ADD_INF 1
////////////////////////////////////////////////////////////////////////////
// Routines to do accurate summation
extern "C" {
static int AS_Compare(const void* px,const void* py)
{
  // Comparison based on absolute values
  double x=fabs(*((const double *)px));
  double y=fabs(*((const double *)py));
  if(x<y) return 1;
  if(x>y) return -1;
  return 0;
}
}

double
AccurateSum(int n,double *arr)
{
  // Order by decreasing magnitude
  qsort(arr,n,sizeof(double),AS_Compare);

  double sum,corr,y,u,t,v,z,x,tmp;

  sum=arr[0]; corr=0;
  for(int i=1;i<n;i++) {
    x=arr[i];
    y=corr+x;
    tmp=y-corr;
    u=x-tmp;
    t=y+sum;
    tmp=t-sum;
    v=y-tmp;
    z=u+v;
    sum=t+z;
    tmp=sum-t;
    corr=z-tmp;
  }
  return sum;
}	

inline double 
Nxx(double x,double y,double z,double a,double b,double c)
{
	return CalculateSDA00(x,y,z,a,b,c);
}

inline double 
Nxy(double x,double y,double z,double a,double b,double c)
{
	return CalculateSDA01(x,y,z,a,b,c);
}

double 
NXXdipole(enum Type_Nij type,double x,double y,double z){
switch(type)
{
	case xx:return Nxxdipole(x,y,z);
   	case yy:return Nxxdipole(y,x,z);
	case zz:return Nxxdipole(z,y,x);
	case xy:return Nxydipole(x,y,z);
	case xz:return Nxydipole(x,z,y);
	case yz:return Nxydipole(y,z,x);
}
}


double 
NXX(enum Type_Nij type,double x,double y,double z,double a,double b,double c)
{
	switch(type){
		case xx:return Nxx(x,y,z,a,b,c);
		case yy:return Nxx(y,x,z,b,a,c);
		case zz:return Nxx(z,y,x,c,b,a);
		case xy:return Nxy(x,y,z,a,b,c);
		case xz:return Nxy(x,z,y,a,c,b);
		case yz:return Nxy(y,z,x,b,c,a);
	}
}

double 
NXXinf(enum Type_Nij type,double x,double y,double z,double X0,double Y0)
{
	switch(type){
		case xx:
		  return Nxxinf(x,y,z,X0,Y0);
		case yy:
		  return Nyyinf(x,y,z,X0,Y0);
		case zz:
		  return Nzzinf(x,y,z,X0,Y0);
		case xy:
        	  return Nxyinf(x,y,z,X0,Y0);
		case xz:
        	  return Nxzinf(x,y,z,X0,Y0);
		case yz:
        	  return Nyzinf(x,y,z,X0,Y0);
	}

}

double getNXXPBC(Type_Nij type,int g,int Nx,int Ny,double x,double y,double z,double a,double b,double c)
{	
	if((type==xy ||type==xz ||type==yz) && x*y==0) return 0.0;

	double Tx=Nx*a,Ty=Ny*b,cof1=1/(4*PI*a*b*c),cof2=a*b*c/(4*PI);
	double* tmpx = new double[2*g+2];
   double* tmpy = new double[2*g+1];
	double tpx,tpy;
	for(int i=-g;i<=g;i++)
	{
		for(int j=-g;j<=g;j++){
			tpx=x+i*Tx;
			tpy=y+j*Ty;
			if(tpx*tpx+tpy*tpy+z*z<10000){
			   tmpy[j+g]=NXX(type,tpx,tpy,z,a,b,c)*cof1;
			}
			else{
			   tmpy[j+g]=NXXdipole(type,tpx,tpy,z)*cof2;
			}
		}
		tmpx[i+g]=AccurateSum(2*g+1,tmpy);
	}

	double X0=(g+0.5)*Tx;
	double Y0=(g+0.5)*Ty;
	if(ADD_INF)
      		tmpx[2*g+1]=NXXinf(type,x,y,z,X0,Y0)*cof2/(Tx*Ty);
	else
		tmpx[2*g+1]=0;

   double result = AccurateSum(2*g+2,tmpx);

   delete[] tmpx;
   delete[] tmpy;

	return result;
}

